# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

do_ad2cp_ahrs <- function(v, ahrs) {
    .Call(`_oce_do_ad2cp_ahrs`, v, ahrs)
}

do_adv_vector_time <- function(vvdStart, vsdStart, vsdTime, vvdhStart, vvdhTime, n, f) {
    .Call(`_oce_do_adv_vector_time`, vvdStart, vsdStart, vsdTime, vvdhStart, vvdhTime, n, f)
}

do_amsr_average <- function(a, b) {
    .Call(`_oce_do_amsr_average`, a, b)
}

do_approx3d <- function(x, y, z, f, xout, yout, zout) {
    .Call(`_oce_do_approx3d`, x, y, z, f, xout, yout, zout)
}

#' Bilinear Interpolation Within a Grid
#'
#' This is used by \code{\link{topoInterpolate}}.
#'
#' @param x vector of x values at which to interpolate
#' @param y vector of y values at which to interpolate
#' @param gx vector of x values for the grid
#' @param gy vector of y values for the grid
#' @param g matrix of the grid values
#'
#' @return vector of interpolated values
bilinearInterp <- function(x, y, gx, gy, g) {
    .Call(`_oce_bilinearInterp`, x, y, gx, gy, g)
}

do_curl1 <- function(u, v, x, y, geographical) {
    .Call(`_oce_do_curl1`, u, v, x, y, geographical)
}

do_curl2 <- function(u, v, x, y, geographical) {
    .Call(`_oce_do_curl2`, u, v, x, y, geographical)
}

do_biosonics_ping <- function(bytes, Rspp, Rns, Rtype) {
    .Call(`_oce_do_biosonics_ping`, bytes, Rspp, Rns, Rtype)
}

do_fill_gap_1d <- function(x, rule) {
    .Call(`_oce_do_fill_gap_1d`, x, rule)
}

do_fill_gap_2d <- function(m, Gap, Debug) {
    .Call(`_oce_do_fill_gap_2d`, m, Gap, Debug)
}

do_gappy_index <- function(starts, offset, length) {
    .Call(`_oce_do_gappy_index`, starts, offset, length)
}

do_geoddist_alongpath <- function(lon, lat, a, f) {
    .Call(`_oce_do_geoddist_alongpath`, lon, lat, a, f)
}

do_geoddist <- function(lon1, lat1, lon2, lat2, a, f) {
    .Call(`_oce_do_geoddist`, lon1, lat1, lon2, lat2, a, f)
}

do_geod_xy <- function(lon, lat, lonr, latr, a, f) {
    .Call(`_oce_do_geod_xy`, lon, lat, lonr, latr, a, f)
}

do_geod_xy_inverse <- function(x, y, lonr, latr, a, f) {
    .Call(`_oce_do_geod_xy_inverse`, x, y, lonr, latr, a, f)
}

do_get_bit <- function(buf, bit) {
    .Call(`_oce_do_get_bit`, buf, bit)
}

do_gradient <- function(m, x, y) {
    .Call(`_oce_do_gradient`, m, x, y)
}

do_interp_barnes <- function(x, y, z, w, xg, yg, xr, yr, gamma, iterations) {
    .Call(`_oce_do_interp_barnes`, x, y, z, w, xg, yg, xr, yr, gamma, iterations)
}

do_landsat_transpose_flip <- function(m) {
    .Call(`_oce_do_landsat_transpose_flip`, m)
}

do_landsat_numeric_to_bytes <- function(m, bits) {
    .Call(`_oce_do_landsat_numeric_to_bytes`, m, bits)
}

do_ldc_ad2cp_in_file <- function(filename, from, to, by, debug) {
    .Call(`_oce_do_ldc_ad2cp_in_file`, filename, from, to, by, debug)
}

do_ldc_rdi_in_file <- function(filename, from, to, by, startIndex, mode, debug) {
    .Call(`_oce_do_ldc_rdi_in_file`, filename, from, to, by, startIndex, mode, debug)
}

do_ldc_rdi_in_file_new <- function(filename, from, to, by, startIndex, mode, debug) {
    .Call(`_oce_do_ldc_rdi_in_file_new`, filename, from, to, by, startIndex, mode, debug)
}

locateByteSequences <- function(buf, match, len, key, max) {
    .Call(`_oce_locateByteSequences`, buf, match, len, key, max)
}

locateVectorImuSequences <- function(buf) {
    .Call(`_oce_locateVectorImuSequences`, buf)
}

mapAssemblePolygons <- function(lon, lat, z) {
    .Call(`_oce_mapAssemblePolygons`, lon, lat, z)
}

mapCheckPolygons <- function(x, y, z, xokspan, usr) {
    .Call(`_oce_mapCheckPolygons`, x, y, z, xokspan, usr)
}

mapClipXy <- function(x, y, usr) {
    .Call(`_oce_mapClipXy`, x, y, usr)
}

match2bytes <- function(buf, m1, m2, demand_sequential) {
    .Call(`_oce_match2bytes`, buf, m1, m2, demand_sequential)
}

match3bytes <- function(buf, m1, m2, m3) {
    .Call(`_oce_match3bytes`, buf, m1, m2, m3)
}

do_matrix_smooth <- function(mat) {
    .Call(`_oce_do_matrix_smooth`, mat)
}

do_oceApprox <- function(x, y, xout, method) {
    .Call(`_oce_do_oceApprox`, x, y, xout, method)
}

do_oce_convolve <- function(x, f, end) {
    .Call(`_oce_do_oce_convolve`, x, f, end)
}

do_oce_filter <- function(x, a, b) {
    .Call(`_oce_do_oce_filter`, x, a, b)
}

do_runlm <- function(x, y, xout, window, L) {
    .Call(`_oce_do_runlm`, x, y, xout, window, L)
}

do_sfm_enu <- function(heading, pitch, roll, starboard, forward, mast) {
    .Call(`_oce_do_sfm_enu`, heading, pitch, roll, starboard, forward, mast)
}

do_ldc_sontek_adp <- function(buf, have_ctd, have_gps, have_bottom_track, pcadp, max) {
    .Call(`_oce_do_ldc_sontek_adp`, buf, have_ctd, have_gps, have_bottom_track, pcadp, max)
}

unwrapSequenceNumbers <- function(seq, bytes) {
    .Call(`_oce_unwrapSequenceNumbers`, seq, bytes)
}

ldcSontekAdv22 <- function(buf, max) {
    .Call(`_oce_ldcSontekAdv22`, buf, max)
}

do_epic_time_to_ymdhms <- function(julianDay, millisecond) {
    .Call(`_oce_do_epic_time_to_ymdhms`, julianDay, millisecond)
}

do_trap <- function(x, y, type) {
    .Call(`_oce_do_trap`, x, y, type)
}

trimTs <- function(x, xlim, extra) {
    .Call(`_oce_trimTs`, x, xlim, extra)
}

