% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ladp.R
\name{summary,ladp-method}
\alias{summary,ladp-method}
\title{Summarize an ladp Object}
\usage{
\S4method{summary}{ladp}(object, ...)
}
\arguments{
\item{object}{an \linkS4class{ladp} object.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix containing statistics of the elements of the \code{data} slot.
}
\description{
Pertinent summary information is presented, including the station name,
sampling location, data ranges, etc.
}
\seealso{
Other things related to ladp data: 
\code{\link{[[,ladp-method}},
\code{\link{[[<-,ladp-method}},
\code{\link{as.ladp}()},
\code{\link{ladp-class}},
\code{\link{plot,ladp-method}}
}
\author{
Dan Kelley
}
\concept{things related to ladp data}
