% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{argShow}
\alias{argShow}
\title{Show a Function Argument}
\usage{
argShow(x, nshow = 4, last = FALSE, sep = "=")
}
\arguments{
\item{x}{the argument}

\item{nshow}{number of values to show at first (if length(x)> 1)}

\item{last}{indicates whether this is the final argument to the function}

\item{sep}{the separator between name and value}
}
\description{
Show a Function Argument
}
