% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{standardizeLongitude}
\alias{standardizeLongitude}
\title{Put Longitude in the Range From -180 to 180}
\usage{
standardizeLongitude(longitude)
}
\arguments{
\item{longitude}{in degrees East, possibly exceeding 180}
}
\value{
longitude in signed degrees East
}
\description{
Put Longitude in the Range From -180 to 180
}
\seealso{
\code{\link[=matrixShiftLongitude]{matrixShiftLongitude()}} and \code{\link[=shiftLongitude]{shiftLongitude()}} are more
powerful relatives to \code{standardizeLongitude}.
}
