% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coastline.R
\name{summary,coastline-method}
\alias{summary,coastline-method}
\alias{summary.coastline}
\title{Summarize a coastline Object}
\usage{
\S4method{summary}{coastline}(object, ...)
}
\arguments{
\item{object}{a \linkS4class{coastline} object.}

\item{\dots}{further arguments passed to or from other methods.}
}
\description{
Summarizes coastline length, bounding box, etc.
}
\seealso{
Other things related to coastline data: 
\code{\link{[[,coastline-method}},
\code{\link{[[<-,coastline-method}},
\code{\link{as.coastline}()},
\code{\link{coastline-class}},
\code{\link{coastlineBest}()},
\code{\link{coastlineCut}()},
\code{\link{coastlineWorld}},
\code{\link{download.coastline}()},
\code{\link{plot,coastline-method}},
\code{\link{read.coastline.openstreetmap}()},
\code{\link{read.coastline.shapefile}()},
\code{\link{subset,coastline-method}}
}
\author{
Dan Kelley
}
\concept{things related to coastline data}
