% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gps.R
\name{summary,gps-method}
\alias{summary,gps-method}
\title{Summarize a gps Object}
\usage{
\S4method{summary}{gps}(object, ...)
}
\arguments{
\item{object}{an object of class \code{"gps"}}

\item{\dots}{further arguments passed to or from other methods.}
}
\description{
Summarize a \linkS4class{gps} object.
}
\seealso{
Other things related to gps data: 
\code{\link{[[,gps-method}},
\code{\link{[[<-,gps-method}},
\code{\link{as.gps}()},
\code{\link{gps-class}},
\code{\link{plot,gps-method}},
\code{\link{read.gps}()}
}
\author{
Dan Kelley
}
\concept{things related to gps data}
