% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClass.R
\name{presentTime}
\alias{presentTime}
\title{Get the Present Time, in a Stated Timezone}
\usage{
presentTime(tz = "UTC")
}
\arguments{
\item{tz}{String indicating the desired timezone. The default is
to use UTC, which is used commonly in oceanographic work. To
get the local time, use \code{tz=""} or \code{tz=NULL}, as in \dQuote{Examples}.}
}
\value{
A \code{\link[=POSIXct]{POSIXct()}}-style object holding the present time, in the
indicated timezone.
}
\description{
Get the Present Time, in a Stated Timezone
}
\examples{
presentTime() # UTC
presentTime("") # the local timezone

}
