% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClass.R
\name{composite,list-method}
\alias{composite,list-method}
\title{Composite by Averaging Across Data}
\usage{
\S4method{composite}{list}(object)
}
\arguments{
\item{object}{a \link{list} of \linkS4class{oce} objects.}
}
\description{
This is done
by calling a specialized version of the function defined in the given
class. In the present
version, the objects must inherit from \linkS4class{amsr}, so the
action is to call
\code{\link[=composite,amsr-method]{composite,amsr-method()}}.

Items within the \code{data} slots of the
objects that are supplied as arguments are averaged in a way that
makes sense for the object class, i.e. taking into account
the particular bad-data codes of that particular class.
}
\seealso{
Other functions that create composite objects: 
\code{\link{composite}()},
\code{\link{composite,amsr-method}}
}
\concept{functions that create composite objects}
