% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/odf.R
\name{ODFListFromHeader}
\alias{ODFListFromHeader}
\title{Create a List of odf Header Metadata}
\usage{
ODFListFromHeader(header)
}
\arguments{
\item{header}{Vector of character strings, holding the header}
}
\value{
A list holding the metadata, with item names matching
those in the ODF header, except that duplicates are transformed
through the use of \code{\link[=unduplicateNames]{unduplicateNames()}}.
}
\description{
Create a List of odf Header Metadata
}
\seealso{
Other things related to odf data: 
\code{\link{CTD_BCD2014666_008_1_DN.ODF.gz}},
\code{\link{ODF2oce}()},
\code{\link{ODFNames2oceNames}()},
\code{\link{[[,odf-method}},
\code{\link{[[<-,odf-method}},
\code{\link{odf-class}},
\code{\link{plot,odf-method}},
\code{\link{read.ctd.odf}()},
\code{\link{read.odf}()},
\code{\link{subset,odf-method}},
\code{\link{summary,odf-method}}
}
\concept{things related to odf data}
