% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argo2ctd.R
\name{argo2ctd}
\alias{argo2ctd}
\title{Coerce argo Data Into a ctd Object}
\usage{
argo2ctd(argo, profile = NULL, debug = getOption("oceDebug"))
}
\arguments{
\item{argo}{an \linkS4class{argo} object.}

\item{profile}{an integer specifying the profile to pick within the argo
object. If this is not provided, it will be set to 1, and a warning
will be issued telling of this setting.}

\item{debug}{an integer specifying whether debugging information is
to be printed during the processing. This is a general parameter that
is used by many \code{oce} functions. Generally, setting \code{debug=0}
turns off the printing, while higher values suggest that more information
be printed. If one function calls another, it usually reduces the value of
\code{debug} first, so that a user can often obtain deeper debugging
by specifying higher \code{debug} values.}
}
\description{
Convert a \linkS4class{argo} object into a \linkS4class{ctd} object.  This
function may be called by \code{\link[=as.ctd]{as.ctd()}} or called directly. In the
first case, note that the only \code{\link[=as.ctd]{as.ctd()}} parameters that are
provided to the present function are the object itself,
the index number of the desired profile, and the debug
value.
}
\details{
The resultant \linkS4class{ctd} object has a constructed station name, made by
taking the filename stored within \code{argo} and appending an underscore
character, followed by the profile number.  For example, if the filename for
\code{argo} is \code{D6902967_001.nc} and if the \code{profile} parameter is 1, then the
station number will be \code{D6902967_001_1}.
}
\examples{
# Read a built-in Argo dataset and convert to a CTD object
# for plotting.
library(oce)
argo <- read.argo(system.file("extdata", "D4902337_219.nc", package = "oce"))
ctd <- as.ctd(argo) # warns of a default 'profile' choice
plot(ctd)

}
\seealso{
Other things related to argo data: 
\code{\link{D4902337_219.nc}},
\code{\link{[[,argo-method}},
\code{\link{[[<-,argo-method}},
\code{\link{argo}},
\code{\link{argo-class}},
\code{\link{argoGrid}()},
\code{\link{argoNames2oceNames}()},
\code{\link{as.argo}()},
\code{\link{handleFlags,argo-method}},
\code{\link{plot,argo-method}},
\code{\link{read.argo}()},
\code{\link{read.argo.copernicus}()},
\code{\link{subset,argo-method}},
\code{\link{summary,argo-method}}

Other things related to ctd data: 
\code{\link{CTD_BCD2014666_008_1_DN.ODF.gz}},
\code{\link{[[,ctd-method}},
\code{\link{[[<-,ctd-method}},
\code{\link{as.ctd}()},
\code{\link{cnvName2oceName}()},
\code{\link{ctd}},
\code{\link{ctd-class}},
\code{\link{ctd.cnv.gz}},
\code{\link{ctdDecimate}()},
\code{\link{ctdFindProfiles}()},
\code{\link{ctdFindProfilesRBR}()},
\code{\link{ctdRaw}},
\code{\link{ctdRepair}()},
\code{\link{ctdTrim}()},
\code{\link{ctd_aml_type1.csv.gz}},
\code{\link{ctd_aml_type3.csv.gz}},
\code{\link{d200321-001.ctd.gz}},
\code{\link{d201211_0011.cnv.gz}},
\code{\link{handleFlags,ctd-method}},
\code{\link{initialize,ctd-method}},
\code{\link{initializeFlagScheme,ctd-method}},
\code{\link{oceNames2whpNames}()},
\code{\link{oceUnits2whpUnits}()},
\code{\link{plot,ctd-method}},
\code{\link{plotProfile}()},
\code{\link{plotScan}()},
\code{\link{plotTS}()},
\code{\link{read.ctd}()},
\code{\link{read.ctd.aml}()},
\code{\link{read.ctd.itp}()},
\code{\link{read.ctd.odf}()},
\code{\link{read.ctd.odv}()},
\code{\link{read.ctd.saiv}()},
\code{\link{read.ctd.sbe}()},
\code{\link{read.ctd.ssda}()},
\code{\link{read.ctd.woce}()},
\code{\link{read.ctd.woce.other}()},
\code{\link{setFlags,ctd-method}},
\code{\link{subset,ctd-method}},
\code{\link{summary,ctd-method}},
\code{\link{woceNames2oceNames}()},
\code{\link{woceUnit2oceUnit}()},
\code{\link{write.ctd}()}
}
\author{
Dan Kelley
}
\concept{things related to argo data}
\concept{things related to ctd data}
